import { Construct } from 'constructs';
import { aws_ec2, aws_iam } from 'aws-cdk-lib';
/**
 * The properties for the bastion instance.
 */
export interface BastiInstanceProps {
    /**
     * The VPC to deploy the bastion instance into.
     */
    readonly vpc: aws_ec2.IVpc;
    /**
     * (Optional) The ID of the bastion instance. The ID will be used to identify
     * the bastion instance. If not specified, a random ID will be generated.
     *
     * @default - A 8-character pseudo-random string
     */
    readonly bastiId?: string;
    /**
     * (Optional) The subnet selection to deploy the bastion instance into.
     * If not specified, the default subnet selection will be used.
     *
     * @default - Public subnets in the VPC
     */
    readonly vpcSubnets?: aws_ec2.SubnetSelection;
    /**
     * (Optional) The instance type to use for the bastion instance.
     *
     * @default t2.micro
     */
    readonly instanceType?: aws_ec2.InstanceType;
    /**
     * (Optional) The machine image to use for the bastion instance.
     */
    readonly machineImage?: aws_ec2.IMachineImage;
}
export interface IBastiInstance {
    /**
     * The bastion instance role.
     */
    readonly role: aws_iam.IRole;
    /**
     * The bastion instance security group.
     */
    readonly securityGroup: aws_ec2.ISecurityGroup;
    /**
     * The ID of the bastion instance.
     */
    readonly bastiId: string;
    /**
     * The VPC the bastion instance is deployed into.
     */
    readonly vpc: aws_ec2.IVpc;
}
/**
 * The basti instance.
 */
export declare class BastiInstance extends Construct implements IBastiInstance {
    /**
     * The bastion instance.
     */
    readonly instance: aws_ec2.Instance;
    /**
     * The bastion instance role.
     */
    readonly role: aws_iam.IRole;
    /**
     * The bastion instance security group.
     */
    readonly securityGroup: aws_ec2.ISecurityGroup;
    /**
     * The ID of the bastion instance.
     */
    readonly bastiId: string;
    /**
     * The VPC the bastion instance is deployed into.
     */
    readonly vpc: aws_ec2.IVpc;
    constructor(scope: Construct, id: string, props: BastiInstanceProps);
    /**
     * Grants an IAM principal permission to connect to the bastion instance.
     * Using the Basti CLI.
     *
     * @param grantee The principal to grant permission to.
     */
    grantBastiCliConnect(grantee: aws_iam.IGrantable): void;
    /**
     * Create a bastion instance from an existing Basti ID.
     *
     * @param scope CDK construct scope
     * @param id CDK construct ID
     * @param bastiId The ID of the basti instance
     * @param vpc The VPC that the bastion is deployed into
     */
    static fromBastiId(scope: Construct, id: string, bastiId: string, vpc: aws_ec2.IVpc): IBastiInstance;
}
